#include<stdio.h>
#include<cuda.h>
#include<cuda_runtime.h>

__global__ void add(int *a, int *b, int *c) {
        *c = *a + *b;
}

// CPU function to generate a vector of random integers
void random_ints (int *a, int n) {
  for (int i = 0; i < n; i++)
  a[i] = rand() % 10000; // random number between 0 and 9999
}

#define N 512

int main(void) { 
        int *a, *b, *c;                 // host copies of a, b, c
        int *d_a, *d_b, *d_c;           // devices copies of a, b, c
        int size = N * sizeof(int);

        // Allow space for device copies of a, b, c
        cudaMalloc((void **)&d_a, size);
        cudaMalloc((void **)&d_b, size);
        cudaMalloc((void **)&d_c, size);

	// Alloc space for host copies of a, b, c and setup input values
	a = (int *)malloc(size); random_ints(a, N);
	b = (int *)malloc(size); random_ints(b, N);
	c = (int *)malloc(size);

	// Copy inputs to device
	cudaMemcpy(d_a, a, size, cudaMemcpyHostToDevice);
	cudaMemcpy(d_b, b, size, cudaMemcpyHostToDevice);

        // Launch add() kernel on GPU with N block
        add<<<N,1>>>(d_a, d_b, d_c);

        // Copy result back to host
        cudaMemcpy(c, d_c, size, cudaMemcpyDeviceToHost);

	// Cleaup
	free(a); free(b); free(c);
	cudaFree(d_a); cudaFree(d_b); cudaFree(d_c);

	return 0;

}

